@@ -47,12 +47,12 @@
 ///////////////////////////  Typedefs  ///////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 
-typedef enum Verbosity_t 
+typedef enum 
 {
    VERBOSITY_NONE, 
    VERBOSITY_SHORT, 
    VERBOSITY_LONG,
-};
+} Verbosity_t;
 
 typedef enum
 {
